#include "PeriodicProject.h"
#include "AdaptStationary.h"
/// Dirichlet boundary function
class G : public AbstractFunction<double, WorldVector<double> >
{
public:
  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    return 0.0;
  }
};

/// RHS function
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {}

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    int dim = x.getSize();
    double r2 = (x*x);
    double ux = exp(-10.0*r2);
    return -(400.0*r2 - 20.0*dim)*ux;
  }
};

Periodicdemo::Periodicdemo():
  periodic("periodic"),
  adaptInfo(NULL),
  adapt(NULL),
  matrixOperator(NULL),
  rhsOperator(NULL)
{}

Periodicdemo::~Periodicdemo() {
  if(matrixOperator != NULL)
    delete matrixOperator;
  if(rhsOperator != NULL)
    delete rhsOperator;
  if(adapt != NULL)
    delete adapt;
  if(adaptInfo != NULL)
    delete adaptInfo;
}

void Periodicdemo::create(string& filename) {
   // ===== init parameters =====
  Parameters::init(false, filename);

  // ===== create and init the scalar problem ===== 
  periodic.initialize(INIT_ALL);

  // === create adapt info ===
  adaptInfo = new AdaptInfo("periodic->adapt", 1);

  // === create adapt ===
  adapt = new AdaptStationary("periodic->adapt",
			       &periodic,
			       adaptInfo);
  
  // ===== create matrix operator =====
  matrixOperator = new Operator(periodic.getFeSpace());
  matrixOperator->addSecondOrderTerm(new Laplace_SOT);
  periodic.addMatrixOperator(matrixOperator);

  // ===== create rhs operator =====
  int degree = periodic.getFeSpace()->getBasisFcts()->getDegree();
  rhsOperator = new Operator(periodic.getFeSpace());
  rhsOperator->addZeroOrderTerm(new CoordsAtQP_ZOT(new F(degree)));
  periodic.addVectorOperator(rhsOperator);

  // ===== add boundary conditions =====
  periodic.addDirichletBC(1, new G);

  // ===== add periodic conditions =====
  periodic.addPeriodicBC(-1);

}

int Periodicdemo::solve(SolutionInformation& info) {
  int retCode = adapt->adapt();
  info.dofVec = periodic.getSolution();
  return retCode;
}
